from io import IOBase

from semantha_sdk.api import SemanthaAPIEndpoint
from semantha_sdk.model import Document


class DocumentAnnotations(SemanthaAPIEndpoint):
    @property
    def _endpoint(self):
        return self._parent_endpoint + "/documentannotations"

    def post_document_annotations(
            self,
            file: IOBase,
            document: Document,
            similarity_threshold: float = 0.85,
            synonymous_threshold: float = 0.98,
            mark_no_match: bool = False,
            with_reference_text: bool = False
    ):
        """ Download the original input document with the referenced document/library matches as annotated comments.

        Args:
            file (IOBase): Input document (left document).
            document (Document): ...
            similarity_threshold (float): Threshold for the similarity score.
                semantha will not deliver results with a sentence score lower than the threshold.
                In general, the higher the threshold, the more precise the results.
            synonymous_threshold (float): Threshold for good matches.
            mark_no_match (bool): Marks paragraphs that have not matched.
            with_reference_text (bool): Provide the reference text in the result JSON.
                If set to false, you have to query the library to resolve the references yourself.
        """
        return self._session.post(
            self._endpoint,
            body={
                "file": file,
                "document": document.data,
                "similaritythreshold": str(similarity_threshold),
                "synonymousthreshold": str(synonymous_threshold),
                "marknomatch": str(mark_no_match),
                "withreferencetext": str(with_reference_text)
            }
        ).execute()



from dataclasses import dataclass

from semantha_sdk.model import SemanthaModelEntity
from semantha_sdk.model.Utils import check_key


@dataclass(frozen=True)
class DocumentMetadata(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict
        check_key(self.data, "fileName", str)
        check_key(self.data, "documentType", str)



from abc import ABC, abstractmethod

from semantha_sdk.rest import RestClient


class SemanthaAPIEndpoint(ABC):
    def __init__(self, session: RestClient, parent_endpoint: str):
        self._session = session
        self._parent_endpoint = parent_endpoint

    @property
    @abstractmethod
    def _endpoint(self):
        pass



from semantha_sdk.model.DomainConfiguration import DomainConfiguration
from semantha_sdk.model.DomainSettings import DomainSettings
from semantha_sdk.model.Domains import Domains as _DomainsDTO, Domain as _DomainDTO
from semantha_sdk.rest.RestClient import RestClient
from semantha_sdk.response.SemanthaPlatformResponse import SemanthaPlatformResponse
from semantha_sdk.api.Documents import Documents
from semantha_sdk.api.ReferenceDocuments import ReferenceDocuments
from semantha_sdk.api.References import References
from semantha_sdk.api.DocumentAnnotations import DocumentAnnotations
from semantha_sdk.api.DocumentComparisons import DocumentComparisons
from semantha_sdk.api import SemanthaAPIEndpoint


class Domain(SemanthaAPIEndpoint):
    """ Endpoint for a specific domain.

        References: documents, documentannotations, documentcomparisons, documentclasses,
            modelclasses, modelinstances, referencedocuments, references,
            settings, stopwords, similaritymatrix, tags and validation.
    """
    def __init__(self, session: RestClient, parent_endpoint: str, domain_name: str):
        super().__init__(session, parent_endpoint)
        self._domain_name = domain_name
        self.__documents = Documents(session, self._endpoint)
        self.__document_annotations = DocumentAnnotations(session, self._endpoint)
        self.__document_comparisons = DocumentComparisons(session, self._endpoint)
        self.__reference_documents = ReferenceDocuments(session, self._endpoint)
        self.__references = References(session, self._endpoint)

    @property
    def _endpoint(self):
        return self._parent_endpoint + f"/{self._domain_name}"

    @property
    def documents(self):
        return self.__documents

    @property
    def reference_documents(self) -> ReferenceDocuments:
        return self.__reference_documents

    @property
    def references(self) -> References:
        return self.__references

    def get_metadata(self) -> SemanthaPlatformResponse:
        return self._session.get(f"{self._endpoint}/metadata").execute()

    def get_rule(self) -> SemanthaPlatformResponse:
        return self._session.get(f"{self._endpoint}/rules").execute()

    def get_documentclasses(self) -> SemanthaPlatformResponse:
        return self._session.get(f"{self._endpoint}/documentclasses").execute()

    def get_subclasses(self, id: str) -> SemanthaPlatformResponse:
        return self._session.get(
            f"{self._endpoint}/documentclasses/{id.lower()}/documentclasses"
        ).execute()

    def get_class_with_subclasses(self, id: str) -> SemanthaPlatformResponse:
        return self._session.get(f"{self._endpoint}/documentclasses/{id.lower()}").execute()

    def get_class_documents(self, id: str) -> SemanthaPlatformResponse:
        return self._session.get(
            f"{self._endpoint}/documentclasses/{id.lower()}/referencedocuments"
        ).execute()

    def get_configuration(self) -> DomainConfiguration:
        return self._session.get(f"{self._endpoint}").execute().to(DomainConfiguration)

    def get_model_classes(self) -> SemanthaPlatformResponse:
        return self._session.get(f"{self._endpoint}/modelclasses").execute()

    def get_settings(self) -> DomainSettings:
        return self._session.get(f"{self._endpoint}/settings").execute().to(DomainSettings)

    def get_stopwords(self) -> SemanthaPlatformResponse:
        return self._session.get(f"{self._endpoint}/stopwords").execute()

    def get_tags(self) -> SemanthaPlatformResponse:
        return self._session.get(f"{self._endpoint}/tags").execute()

    def get_custom_entity_by_id(self, domain: str, entity: str):
        return self._session.get(f"/api/model/domains/{domain}/namedentities/{entity}").execute()

    def get_all_regexes(self, domain: str):
        return self._session.get(f"/api/model/domains/{domain}/regexes").execute()

    def get_regex(self, domain: str, id: str):
        return self._session.get(f"/api/model/domains/{domain}/regexes/{id}").execute()

    def get_all_synonyms(self, domain: str):
        return self._session.get(f"/api/model/domains/{domain}/synonyms").execute()


# TODO: Add docstrings, comments, type hints and error handling.
class Domains:
    """
        References:
            Specific domains by name
    """
    def __init__(self, session: RestClient, parent_endpoint: str):
        self._session = session
        self.__endpoint = parent_endpoint + "/domains"

    def get_all(self) -> list[_DomainDTO]:
        """ Get all available domains """
        return self._session.get(self.__endpoint).execute().to(_DomainsDTO).domains

    def get_one(self, domain_name: str) -> Domain:
        # Returns a Domain object for the given domainname, throws error if id doesn't exist
        return Domain(self._session, self.__endpoint, domain_name)



from dataclasses import dataclass

from semantha_sdk.model import SemanthaModelEntity, Paragraph


@dataclass(frozen=True)
class PageContent(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def paragraphs(self) -> list[Paragraph]:
        if "paragraphs" in self.data:
            return [
                Paragraph(raw_paragraph) for raw_paragraph in self.data["paragraphs"]
            ]
        else:
            return []

    # TODO tables DTO
    @property
    def tables(self):
        return None

    # TODO annotation page DTO
    @property
    def annotation_page(self):
        return None


class Page(SemanthaModelEntity):

    # def __init__(self, data: Union[dict, list]):
    #     super().__init__(data)

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def contents(self) -> list[PageContent]:
        return [PageContent(raw_content) for raw_content in self.data["contents"]]



from semantha_sdk.rest import RestClient
from semantha_sdk.api import SemanthaAPIEndpoint
from semantha_sdk.response import SemanthaPlatformResponse
from semantha_sdk.model import Class as ClassDTO


class Class(SemanthaAPIEndpoint):
    """ Endpoint for a specific class.

        Allows for accessing class instances.
    """

    def __init__(self, session: RestClient, parent_endpoint: str, classid: str):
        super().__init__(session, parent_endpoint)
        self.__classid = classid

    @property
    def _endpoint(self):
        return self._parent_endpoint + f"/{self.__classid}"

    def get_instance(self):
        """ Get all instances of the class """
        return self.__sesssion.get(self._endpoint).execute()

    def delete_instance(self):
        """ Delete all instances of the class """
        return self.__sesssion.delete(self._endpoint).execute()


class Classes(SemanthaAPIEndpoint):
    """ Endpoint for the classes in a domain.

        References: Specific api for specific classes

    """

    @property
    def _endpoint(self):
        return self._parent_endpoint + "/classes"

    def get_classes(self) -> SemanthaPlatformResponse:
        """ Get all classes """
        return self._session.get(self._endpoint).execute()

    def post_classes(self, classes: list[ClassDTO]) -> SemanthaPlatformResponse:
        """ Create one or more classes """
        body = [_class.data for _class in classes]
        return self._session.post(self._endpoint, body).execute()

    def delete_classes(self) -> SemanthaPlatformResponse:
        """ Delete all classes """
        return self._session.delete(self._endpoint).execute()

    def get_class(self, classid: str):
        """ Get a specific class by id """
        return Class(self._session, self._endpoint, classid)

    def put_class(self, _class: ClassDTO):
        """ Add a given class """
        body = _class.data
        return self._session.put(self._endpoint, body).execute()

    def delete_class(self, classid: str):
        """ Delete a class given its class id """
        return self._session.delete(self._endpoint + f"/{classid}").execute()



from semantha_sdk.api import SemanthaAPIEndpoint
from semantha_sdk.api.CurrentUser import CurrentUser
from semantha_sdk.api.Diff import Diff
from semantha_sdk.api.Domains import Domains
from semantha_sdk.api.Model import Model
from semantha_sdk.rest.RestClient import RestClient


class SemanthaAPI(SemanthaAPIEndpoint):
    """ Entry point to the Semantha API.

        References the /currentuser, /domains and /diff endpoints.

        Note:
            The __init__ method is not meant to be invoked directly
            use `semantha_sdk.login()` with your credentials instead.
    """

    def __init__(self, session: RestClient, parent_endpoint: str):
        super().__init__(session, parent_endpoint)
        self.__current_user = CurrentUser(session, self._endpoint)
        self.__diff = Diff(session, self._endpoint)
        self.__domains = Domains(session, self._endpoint)
        self.__model = Model(session, self._endpoint)

    @property
    def _endpoint(self):
        return self._parent_endpoint

    @property
    def current_user(self) -> CurrentUser:
        return self.__current_user

    @property
    def domains(self) -> Domains:
        return self.__domains

    @property
    def diff(self) -> Diff:
        return self.__diff

    @property
    def model(self) -> Model:
        return self.__model



from requests import Request

from semantha_sdk.request import SemanthaRequest


class RestClient:

    def __init__(self, server_url: str, api_key: str):
        self.__server_url = server_url
        self.__api_key = api_key

    def __build_headers_for_json_request(self) -> dict[str, str]:
        return {
            'Accept': 'application/json',
            'Authorization': f'Bearer {self.__api_key}'
        }

    def __request(self,
                  method,
                  url,
                  headers=None,
                  files=None,
                  data=None,
                  params=None,
                  auth=None,
                  cookies=None,
                  hooks=None,
                  json=None
                  ) -> SemanthaRequest:
        if headers is None:
            headers = self.__build_headers_for_json_request()
        request = Request(
            method=method,
            url=self.__server_url + url,
            headers=headers,
            files=files,
            data=data,
            params=params,
            auth=auth,
            cookies=cookies,
            hooks=hooks,
            json=json
        )
        prepared_request = request.prepare()
        return SemanthaRequest(prepared_request)

    def get(self, url: str, q_params: dict[str, str] = None) -> SemanthaRequest:
        return self.__request("GET", url, params=q_params)

    def post(self, url: str, body: dict, q_params: dict[str, str] = None) -> SemanthaRequest:
        return self.__request("POST", url, files=body, params=q_params)

    # TODO: Needs to accept list of dicts as well!
    def delete(self, url: str, q_params: dict[str, str] = None) -> SemanthaRequest:
        return self.__request("DELETE", url, params=q_params)

    # TODO: Implement PUT and PATCH
    def patch(self, url: str, body: dict, q_params: dict[str, str] = None) -> SemanthaRequest:
        raise NotImplementedError("Patch requests aren't possible yet")

    def put(self, url: str, body: dict, q_params: dict[str, str] = None) -> SemanthaRequest:
        raise NotImplementedError("Put requests aren't possible yet")



from io import IOBase

from semantha_sdk.api import SemanthaAPIEndpoint
from semantha_sdk.model.Document import Document
from semantha_sdk.model import DocumentMetadata


class References(SemanthaAPIEndpoint):

    @property
    def _endpoint(self):
        return self._parent_endpoint + "/references"

    def post(
            self,
            file: IOBase = None,
            reference_document: IOBase = None,
            reference_document_ids: list[str] = None,
            tags: str = "",
            document_class_ids: list[str] = None,
            similarity_threshold: float = 0.85,
            synonymous_threshold: float = 0.98,
            mark_no_match: bool = False,
            with_reference_text: bool = False,
            language: str = None,
            mode: str = "fingerprint",
            document_type: str = None,
            meta_data: list[DocumentMetadata] = None,
            with_context: bool = True,
            consider_text_type: bool = False,
            resize_paragraphs: bool = False,
            text: str = None,
            with_areas: bool = False,
            detect_language: bool = False,
            max_references: int = 50
    ) -> Document:
        """ Matches one input document to a set of reference documents.

            If you match against internal library the 'tags' parameter can be used to filter the library.

            Args:
                file (str): Input document (left document).
                reference_document (str): Reference document(s) to be used instead of the documents in the domain's library.
                reference_document_ids (list[str]): To filter for document IDs. The limit here is 1000 IDs.
                    The IDs are passed as a JSON array.
                tags (str): List of tags to filter the reference library.
                    You can combine the tags using a comma (OR) and using a plus sign (AND).
                document_class_ids (list[str]): List of documentclass IDs for the target.
                    The limit here is 1000 IDs. The IDs are passed as a JSON array.
                    This does not apply on the GET referencedocuments call. Here the ids are separated with a comma.
                similarity_threshold (float): Threshold for the similarity score.
                    semantha will not deliver results with a sentence score lower than the threshold.
                    In general, the higher the threshold, the more precise the results.
                synonymous_threshold (float): Threshold for good matches.
                mark_no_match (bool): Marks the matches that have not matched.
                with_reference_text (bool): Provide the reference text in the result JSON.
                    If set to false, you have to query the library to resolve the references yourself.
                language (str): The language of the input document (only available if configured for the domain).
                mode (str):
                    Determine references:
                        Mode to enable if a semantic search ("fingerprint") or keyword search ("keyword") should be considered.
                    Creating document model:
                        It also defines what structure should be considered for what operator (similarity or extraction).
                    One of "fingerprint", "keyword", "document" or "auto".
                document_type (str):
                    Specifies the document type that is to be used by semantha when reading the uploaded PDF document.
                meta_data (list[DocumentMetadataDTO]): Filter documents by metadata.
                with_context (bool): Creates and saves the context.
                consider_text_type (bool):
                    Use this parameter to ensure that only paragraphs of the same type are compared with each other.
                    The parameter is of type boolean and is set to false by default.
                resize_paragraphs (bool): Automatically resizes paragraphs based on their semantic meaning.
                text (str): Plain text input (left document). If set, the parameter file will be ignored.
                with_areas (bool): Gives back the coordinates of referenced area.
                detect_language (bool):
                    Auto-detect the language of the document (only available if configured for the domain).
                max_references (int): Maximum number of returned results.
        """
        response = self._session.post(
            self._endpoint,
            body={
                "file": file,
                "referencedocument": reference_document,
                "referencedocumentids": reference_document_ids,
                "tags": tags,
                "documentclassids": document_class_ids,
                "similaritythreshold": str(similarity_threshold),
                "synonymousthreshold": str(synonymous_threshold),
                "marknomatch": str(mark_no_match),
                "withreferencetext": str(with_reference_text),
                "language": language,
                "mode": mode,
                "documenttype": document_type,
                "metadata": meta_data,
                "withcontext": str(with_context),
                "considertexttype": str(consider_text_type),
                "resizeparagraphs": str(resize_paragraphs),
                "text": text,
                "withareas": str(with_areas)
            },
            q_params={
                "detectlanguage": str(detect_language),
                "maxreferences": str(max_references)
            }
        ).execute()
        return response.to(Document)



from dataclasses import dataclass

from semantha_sdk.model import SemanthaModelEntity


@dataclass(frozen=True)
class Label(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict
        assert type(self.data["lang"]) is str and type(self.data["value"]) is str


from io import IOBase
from typing import Optional

from semantha_sdk.api import SemanthaAPIEndpoint
from semantha_sdk.model.Document import Document
from semantha_sdk.model.Entity import Entity
from semantha_sdk.model.NamedEntities import NamedEntities
from semantha_sdk.model.Paragraphs import Paragraph
from semantha_sdk.model.ReferenceDocuments import ReferenceDocuments as _ReferenceDocumentsDTO, \
    ReferenceDocumentCollection, Statistic
from semantha_sdk.model.Sentences import Sentence
from semantha_sdk.response import SemanthaPlatformResponse


class ReferenceDocuments(SemanthaAPIEndpoint):
    @property
    def _endpoint(self):
        return self._parent_endpoint + "/referencedocuments"

    def get_all(self,
                offset: int = None,
                limit: int = None,
                filter_tags: str = None,
                filter_document_class_ids: str = None,
                filter_name: str = None,
                filter_created_before: int = None,
                filter_created_after: int = None,
                filter_metadata: str = None,
                filter_comment: str = None,
                sort_by: str = None,
                return_fields: str = None) -> _ReferenceDocumentsDTO:
        """ Get reference documents (library documents)

        If no parameters are set all reference documents are returned.
        However, the result set can be filtered (filter_*), sorted (sort_by), sliced (offset AND limit) and the returned
        attributes/fields can be manipulated (return_fields) to reduce the size of the response.
        Note that some filters and sorting can only be used iff slicing is used (offset and limit).

        Args:
            offset (int): the start index (inclusive) of the returned slice of reference documents
            limit (int): the end index (exclusive) of the returned slice of reference documents
            filter_tags (str): the tags to filter by: comma separated lists are interpreted as OR and + is interpreted
                               as AND. E.g. 'a+b,c+d' means a AND b OR c AND d. The tag filter can be used without
                               slicing.
            filter_document_class_ids (str): the class ids to filter by. Filtering by class ids can be done without
                                             slicing.
            filter_name (str): filter by (document) name. Can only be used with slicing (offset and limit).
            filter_created_before (int): filter by creation date before. Can only be used with slicing (offset and
                                         limit).
            filter_created_after (int): filter by creation date after. Can only be used with slicing (offset and
                                        limit).
            filter_metadata (str): filter by metadata. Can only be used with slicing (offset and limit).
            filter_comment (str): filter by comment. Can only be used with slicing (offset and limit).
            sort_by (str): (lexically) sort the result by one or more criteria: "name", "filename", "metadata",
                           "created", "updated", "color", "comment", "derivedcolor", "derivedcomment", "documentclass".
                           If a value is prefixed by a '-', the sorting is inverted. Can only be used with slicing
                           (offset and limit). Note that sorting is performed before slicing.
            return_fields (str): limit the returned fields to the defined (instead of a full response): "id", "name",
                                 "tags", "derivedtags", "metadata", "filename", "created", "processed", "lang",
                                 "updated", "color", "derivedcolor", "comment", "derivedcomment", "documentclass"
        """
        if (offset is None and limit is not None) or (limit is None and offset is not None):
            raise ValueError("'limit' and 'offset' must be set together.")
        if offset is None and limit is None:
            if filter_name is not None:
                raise ValueError("filter by name can only be used if 'limit' and 'offset' are set")
            if filter_created_before is not None:
                raise ValueError("filter by 'created before' can only be used if 'limit' and 'offset' are set")
            if filter_created_after is not None:
                raise ValueError("filter by 'created after' can only be used if 'limit' and 'offset' are set")
            if filter_metadata is not None:
                raise ValueError("filter by metadata can only be used if 'limit' and 'offset' are set")
            if filter_comment is not None:
                raise ValueError("filter by comment can only be used if 'limit' and 'offset' are set")
            if sort_by is not None:
                raise ValueError("sorting can only activated if 'limit' and 'offset' are set")

        q_params = {}
        if filter_tags is not None:
            q_params["tags"] = filter_tags
        if filter_document_class_ids is not None:
            q_params["document_class_ids"] = filter_document_class_ids
        if return_fields is not None:
            q_params["fields"] = return_fields
        if offset is not None and limit is not None:
            q_params["offset"] = offset
            q_params["limit"] = limit
            if filter_name is not None:
                q_params["name"] = filter_name
            if filter_created_before is not None:
                q_params["createdbefore"] = filter_created_before
            if filter_created_after is not None:
                q_params["createdafter"] = filter_created_after
            if filter_metadata is not None:
                q_params["metadata"] = filter_metadata
            if filter_comment is not None:
                q_params["comment"] = filter_comment
            if sort_by is not None:
                q_params["sort"] = sort_by

        return self._session.get(self._endpoint, q_params=q_params).execute().to(_ReferenceDocumentsDTO)

    def delete_all(self):
        """ Delete all reference documents """
        self._session.delete(self._endpoint).execute()

    def post(
        self,
        name: str = None,
        tags: str = None,
        metadata: str = None,
        file: IOBase = None,
        text: str = None,
        document_type: str = None,
        color: str = None,
        comment: str = None,
        detect_language: bool = False
    ) -> ReferenceDocumentCollection:
        """ Upload a reference document

        Args:
            name (str): The document name in your library (in contrast to the file name being used during upload).
            tags (str): List of tags to filter the reference library.
                You can combine the tags using a comma (OR) and using a plus sign (AND).
            metadata (str): Filter by metadata
            file (str): Input document (left document).
            text (str): Plain text input (left document). If set, the parameter file will be ignored.
            document_type (str): Specifies the document type that is to be used when reading the uploaded PDF document. 
            color (str): Use this parameter to specify the color for your reference document.
                Possible values are RED, MAGENTA, AQUA, ORANGE, GREY, or LAVENDER.
            comment (str): Use this parameter to add a comment to your reference document.
            detect_language (bool): Auto-detect the language of the document (only available if configured for the domain).
        """
        return self._session.post(
            self._endpoint,
            body={
                "name": name,
                "tags": tags,
                "metadata": metadata,
                "file": file,
                "text": text,
                "documenttype": document_type,
                "color": color,
                "comment": comment
            },
            q_params={
                "detectlanguage": str(detect_language)
            }
        ).execute().to(ReferenceDocumentCollection)

    def get_one(self, document_id: str) -> Document:
        """ Get a reference document by id """
        return self._session.get(f"{self._endpoint}/{document_id}").execute().to(Document)

    def delete_one(self, document_id: str):
        """ Delete a reference document by id """
        self._session.delete(f"{self._endpoint}/{document_id}").execute()

    def post_update(
            self,
            document_id: str,
            id: str,
            name: str,
            tags: list[str],
            metadata: str,
            file_name: str,
            created: int,
            updated: int,
            processed: bool,
            lang: str,
            content: str,
            documentClass: Entity,
            derived_tags: list[str],
            color: str,
            derived_color: str,
            comment: str,
            derived_comment: str,
            query_by_name: bool = False
    ) -> SemanthaPlatformResponse:
        """ Update reference document by id (not yet implemented)"""
        # TODO implement and adapt DocString
        raise NotImplementedError("Reference documents can not be updated yet")
        return self._session.patch(f"{self._endpoint}/{document_id}").execute()

    def get_paragraph(self, document_id: str, paragraph_id: str) -> Paragraph:
        """ Get a paragraph of a reference document by document id and paragraph id"""
        return self._session.get(f"{self._endpoint}/{document_id}/paragraphs/{paragraph_id}").execute().to(Paragraph)

    def delete_paragraph(self, document_id: str, paragraph_id: str):
        """ Delete a paragraph of a reference document by document id and paragraph id"""
        self._session.delete(f"{self._endpoint}/{document_id}/paragraphs/{paragraph_id}").execute()

    def patch_paragraph(self, document_id: str, paragraph_id: str) -> SemanthaPlatformResponse:
        """(not yet implemented)"""
        # TODO implement and adapt DocString
        return self._session.patch(f"{self._endpoint}/{document_id}/paragraphs/{paragraph_id}").execute()

    def get_sentence(self, document_id: str, sentence_id: str) -> Sentence:
        """ Get a sentence of a reference document by document id and sentence id"""
        return self._session.get(f"{self._endpoint}/{document_id}/sentences/{sentence_id}").execute().to(Sentence)

    def get_clusters(self) -> SemanthaPlatformResponse:
        """(not yet implemented)"""
        # TODO implement and adapt DocString
        raise NotImplementedError("Clustering of reference documents can not be activated via the SDK yet")
        return self._session.get(f"{self._endpoint}/clusters").execute()

    def get_named_entities(self) -> Optional[NamedEntities]:
        """ Get all named entities (aka custom entities) that were extracted from the reference documents.
        Note: Might be None iff no named entities have been extracted.
        """
        return self._session.get(f"{self._endpoint}/namedentities").execute().to(NamedEntities)

    def get_statistic(self) -> Statistic:
        """ Get statistics for reference documents"""
        return self._session.get(f"{self._endpoint}/statistic").execute().to(Statistic)



from enum import Enum

class DataType(Enum):
    @classmethod
    def list(cls):
        return list(map(lambda c: c.value, cls))

    STRING = "STRING"
    CURRENCY = "CURRENCY"
    DATE = "DATE"
    NUMBER = "NUMBER"
    YEAR = "YEAR"
    BOOLEAN = "BOOLEAN"
    INTEGER = "INTEGER"



from semantha_sdk.api import SemanthaAPIEndpoint
from semantha_sdk.rest.RestClient import RestClient


class DataProperties(SemanthaAPIEndpoint):

    @property
    def _endpoint(self):
        return self._parent_endpoint + "/dataproperties"

    def get_data_properties(self, domain: str):
        return self._session.get(f"/api/domains/{domain.lower()}/dataproperties").execute()

    def post_data_properties(self, domain: str, body: dict):
        return self._session.post(f"/api/domains/{domain.lower()}/dataproperties", body).execute()



from dataclasses import dataclass
from typing import List

from semantha_sdk.model import SemanthaModelEntity


@dataclass(frozen=True)
class Domain:
    id: str
    name: str
    base_url: str


@dataclass(frozen=True)
class Domains(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is list

    @property
    def domains(self) -> List[Domain]:
        return [Domain(domain["id"], domain["name"], base_url=domain["baseUrl"]) for domain in self.data]



from dataclasses import dataclass

from semantha_sdk.model import SemanthaModelEntity


@dataclass(frozen=True)
class UserData(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def name(self) -> str:
        return str(self.data["name"])

    @property
    def valid_until(self) -> int:
        return int(self.data["validUntil"])

    @property
    def roles(self) -> list[str]:
        return self.data['roles']



from .SemanthaAPIEndpoint import SemanthaAPIEndpoint
from .Boostwords import Boostwords
from .Classes import Classes, Class
from .CurrentUser import CurrentUser
from .Dataproperties import DataProperties
from .Domains import Domains, Domain
from .Instances import Instances
from .ReferenceDocuments import ReferenceDocuments
from .Documents import Documents
from .SemanthaAPI import SemanthaAPI
from .Diff import Diff



from dataclasses import dataclass

from semantha_sdk.model import SemanthaModelEntity, Label, Metadata, DataType


@dataclass(frozen=True)
class Class(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data["id"]) is str
        assert type(self.data["name"]) is str
        assert type(self.data["readOnly"]) is bool
        assert type(self.data["functional"]) is bool
        assert type(self.data["labels"]) is list[dict]
        assert type(self.data["metadata"]) is list[dict]
        assert type(self.data["comment"]) is str
        assert self.data["datatype"] in DataType.list()
        assert type(self.data["attributeIds"]) is list[str]
        assert type(self.data["relevantForRelation"]) is bool
        assert type(self.data["objectPropertyId"]) is str
        assert type(self.data["parentId"]) is str

    @property
    def labels(self) -> list[Label]:
        return [
            Label(label)
            for label in self.data.get("labels", [])
        ]

    @property
    def metadata(self) -> list[Metadata]:
        return [
            Metadata(values) for values in self.data.get("metadata", [])
        ]



from semantha_sdk.api import SemanthaAPIEndpoint
from semantha_sdk.model.UserData import UserData


class CurrentUser(SemanthaAPIEndpoint):
    """ Access information about the currently logged-in user. """

    @property
    def _endpoint(self):
        return self._parent_endpoint + "/currentuser"

    def get_user_data(self) -> UserData:
        return self._session.get(self._endpoint).execute().to(UserData)

    def get_user_roles(self) -> list[str]:
        return self._session.get(self._endpoint + "/roles").execute().as_list()



from dataclasses import dataclass

from semantha_sdk.model import SemanthaModelEntity


@dataclass(frozen=True)
class DomainSettings(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def similarity_model_id(self) -> str:
        return str(self.data["similarityModelId"])

    @property
    def keep_numbers(self) -> bool:
        return bool(self.data["keepNumbers"])

    @property
    def min_tokens(self) -> int:
        return int(self.data["minTokens"])

    @property
    def similarity_measure(self) -> str:
        return str(self.data["similarityMeasure"])

    @property
    def context_weight(self) -> float:
        return float(self.data["contextWeight"])

    @property
    def enable_string_comparison(self) -> bool:
        return bool(self.data["enableStringComparison"])

    @property
    def default_document_type(self) -> str:
        return str(self.data["defaultDocumentType"])

    @property
    def enable_paragraph_sorting(self) -> bool:
        return bool(self.data["enableParagraphSorting"])

    @property
    def enable_paragraph_end_detection(self) -> bool:
        return bool(self.data["enableParagraphEndDetection"])

    @property
    def enable_paragraph_merging_based_on_formatting(self) -> bool:
        return bool(self.data["enableParagraphMergingBasedOnFormatting"])

    @property
    def enable_boost_word_filtering_for_input_documents(self) -> bool:
        return bool(self.data["enableBoostWordFilteringForInputDocuments"])

    @property
    def tagging_similarity_mode(self) -> str:
        return str(self.data["taggingSimilarityMode"])

    @property
    def enable_updating_fingerprints_on_tag_updates(self) -> bool:
        return bool(self.data["enableUpdatingFingerprintsOnTagUpdates"])

    @property
    def similarity_matcher(self) -> str:
        return str(self.data["similarityMatcher"])

    @property
    def similarity_max_deviation(self) -> int:
        return int(self.data["similarityMaxDeviation"])

    @property
    def similarity_threshold(self) -> float:
        return float(self.data["similarityThreshold"])

    @property
    def enable_tagging(self) -> bool:
        return bool(self.data["enableTagging"])

    @property
    def tagging_threshold(self) -> float:
        return float(self.data["taggingThreshold"])

    @property
    def tagging_strategy(self) -> str:
        return str(self.data["taggingStrategy"])

    @property
    def extraction_threshold(self) -> float:
        return float(self.data["extractionThreshold"])

    @property
    def extraction_strategy(self) -> str:
        return str(self.data["extractionStrategy"])

    @property
    def resize_paragraphs_on_extraction(self) -> bool:
        return bool(self.data["resizeParagraphsOnExtraction"])



from semantha_sdk.api import SemanthaAPIEndpoint
from semantha_sdk.api.Boostwords import Boostwords
from semantha_sdk.rest.RestClient import RestClient


class DomainModel(SemanthaAPIEndpoint):

    def __init__(self, session: RestClient, parent_endpoint: str, domain_name: str):
        super().__init__(session, parent_endpoint)
        self._domain_name = domain_name
        self.__boostwords = Boostwords(session, self._endpoint)

    @property
    def _endpoint(self):
        return self._parent_endpoint + f"/{self._domain_name}"

    @property
    def boostwords(self):
        return self.__boostwords


class Domains:

    def __init__(self, session: RestClient, parent_endpoint: str):
        self._session = session
        self.__endpoint = parent_endpoint + "/domains"

    def get_one(self, domain_name: str) -> DomainModel:
        # Returns a Domain object for the given domainname, throws error if id doesn't exist
        return DomainModel(self._session, self.__endpoint, domain_name)



from dataclasses import dataclass
from typing import Optional

from semantha_sdk.model import SemanthaModelEntity, Reference


@dataclass(frozen=True)
class Sentence(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def id(self) -> str:
        return self.data["id"]

    @property
    def text(self) -> str:
        return self.data["text"]

    @property
    def document_name(self) -> Optional[str]:
        return None

    @property
    def named_entities(self):
        return None

    @property
    def references(self) -> Optional[list[Reference]]:
        return [
            Reference(raw_reference)
            for raw_reference in self.data.get("references", [])
        ]

    @property
    def areas(self):
        return None



from semantha_sdk.api import SemanthaAPIEndpoint
from semantha_sdk.response import SemanthaPlatformResponse


class Boostwords(SemanthaAPIEndpoint):
    @property
    def _endpoint(self):
        return self._parent_endpoint + "/boostwords"

    def get_all(self) -> list:
        """ Get all boostwords """
        return self._session.get(self._endpoint).execute().as_list()

    def post(self, id: str, word: str, regex: str, tags: list[str], label: str) -> SemanthaPlatformResponse:
        """ Create a boostword (not yet implemented)"""
        # TODO implement and adapt DocString
        raise NotImplementedError("Boostwords can not be added yet")
        body = locals()
        return self._session.post(self._endpoint, body).execute()

    def delete_all(self):
        """ Delete all boostwords """
        self._session.delete(self._endpoint).execute()

    def get_one(self, id: str) -> dict:
        """ Get a boostword by id """
        return self._session.get(self._endpoint + f"/{id}").execute().as_dict()

    def put(self, id: str) -> SemanthaPlatformResponse:
        """ Update a boostword by id (not yet implemented)"""
        # TODO implement and adapt DocString
        return self._session.put(self._endpoint + f"/{id}").execute()

    def delete_one(self, id: str):
        """ Delete a boostword by id """
        self._session.delete(self._endpoint + f"/{id}").execute()



from semantha_sdk.rest.RestClient import RestClient


class Instances:
    def __init__(self, session: RestClient, api_endpoint: str):
        self.__session = session
        self.__api_endpoint = api_endpoint

    def get_instances(self):
        return self.__session.get(self.__api_endpoint).execute()

    def post_domain_classes(self, body: dict):
        return self.__session.post(self.__api_endpoint, body).execute()




from dataclasses import dataclass

from semantha_sdk.model import SemanthaModelEntity


@dataclass(frozen=True)
class Metadata(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict
        assert "id" in self.data.keys() and "value" in self.data.keys()
        assert type(self.data["id"]) is str and type(self.data["value"]) is str



from semantha_sdk.api import SemanthaAPIEndpoint
from semantha_sdk.response import SemanthaPlatformResponse


class Documents(SemanthaAPIEndpoint):
    """ /api/{domainname}/documents endpoint. """

    @property
    def _endpoint(self):
        return self._parent_endpoint + "/documents"

    def create_document_model(
            self,
            file: str = None,
            text: str = None,
            _type: str = "similarity",
            document_type: str = None,
            with_areas: bool = False,
            with_context: bool = True,
            mode: str = "sentence",
            with_paragraph_type: bool = False
    ) -> SemanthaPlatformResponse:
        """ Create a document model

        Args:

            file (str): Input document (left document)
            text (str): Plain text input (left document). If set, the parameter file will be ignored.
            _type (str): Enum: "similarity" "extraction". Choose the structure of a document
                for similarity or extraction. The type depends on the Use Case you're in.
            document_type (str): Specifies the document type that is to be used by semantha
                when reading the uploaded PDF document.
            with_areas (bool): Gives back the coordinates of referenced area.
            with_context (bool): Creates and saves the context.
            mode (str): Determine references: Mode to enable if a semantic search (fingerprint)
                or keyword search (keyword) should be considered. Creating document model: It also
                defines what structure should be considered for what operator (similarity or extraction).
            with_paragraph_type (bool): The type of the paragraph, for example heading, text.
        """
        return self._session.post(
            self._endpoint,
            body={
                "file": file,
                "text": text,
                "type": _type,
                "documenttype": document_type,
                "withareas": str(with_areas),
                "withcontext": str(with_context),
                "mode": mode,
                "with_paragraph_type": str(with_paragraph_type)
            }
        ).execute()



from dataclasses import dataclass

from semantha_sdk.model import SemanthaModelEntity


@dataclass(frozen=True)
class NamedEntity(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def name(self):
        return str(self.data["name"])

    @property
    def text(self):
        return str(self.data["text"])


@dataclass(frozen=True)
class NamedEntities(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is list

    def __getitem__(self, index) -> NamedEntity:
        return NamedEntity(self.data[index])



from dataclasses import dataclass
from typing import Optional

from semantha_sdk.model import Reference, Page
from semantha_sdk.model.ReferenceDocuments import DocumentInformation


@dataclass(frozen=True)
class Document(DocumentInformation):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def pages(self) -> list[Page]:
        return [Page(raw_page) for raw_page in self.data["pages"]]

    @property
    def references(self) -> Optional[list[Reference]]:
        return [
            Reference(raw_reference)
            for raw_reference in self.data.get("references", [])
        ]

    @property
    def image_pages(self) -> Optional[str]:
        return None



from dataclasses import dataclass
from typing import Optional

from semantha_sdk.model import SemanthaModelEntity


@dataclass(frozen=True)
class Reference(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def document_id(self) -> str:
        return self.data["documentId"]
        pass

    @property
    def similarity(self) -> float:
        return float(self.data["similarity"])

    @property
    def color(self) -> str:
        return self.data["color"]

    @property
    def document_name(self) -> Optional[str]:
        return self.data.get("documentName", None)

    def page_numer(self) -> Optional[int]:
        return (
            int(self.data["pageNumber"])
            if "pageNumber" in self.data
            else None
        )

    @property
    def paragraph_id(self) -> Optional[str]:
        return self.data.get("paragraphId", None)

    @property
    def sentence_id(self) -> Optional[str]:
        return self.data.get("sentenceId", None)

    @property
    def text(self) -> Optional[str]:
        return self.data.get("text", None)

    @property
    def context(self) -> Optional[dict[str, str]]:
        return self.data.get("context", None)

    @property
    def type(self) -> Optional[str]:
        return self.data.get("type", None)

    @property
    def comment(self) -> Optional[str]:
        return self.data.get("comment", None)

    @property
    def has_opposite_meaning(self) -> Optional[bool]:
        return (
            bool(self.data.get("hasOppositeMeaning", None))
            if "hasOppositeMeaning" in self.data
            else None
        )



from requests import PreparedRequest, Session

from semantha_sdk.response.SemanthaPlatformResponse import SemanthaPlatformResponse


class SemanthaRequest:

    def __init__(self, prepared_request: PreparedRequest):
        self.__prepared_request = prepared_request

    def execute(self) -> SemanthaPlatformResponse:
        with (Session()) as session:
            response = session.send(self.__prepared_request)
            return SemanthaPlatformResponse(response)

    def execute_async(self) -> SemanthaPlatformResponse:
        pass



from dataclasses import dataclass
from typing import Optional

from semantha_sdk.model import SemanthaModelEntity
from semantha_sdk.model.Entity import Entity


@dataclass(frozen=True)
class Parameters(SemanthaModelEntity):
    # TODO: add remaining information whhich is created if we use sorting, filtering or return fields
    #       in reference documents get_all()

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def domain(self) -> str:
        return str(self.data["domain"])


@dataclass(frozen=True)
class Meta(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def parameters(self) -> Parameters:
        return Parameters(self.data["parameters"])


@dataclass(frozen=True)
class DocumentInformation(SemanthaModelEntity):
    # TODO: make all (?) properties optional since they might not be present if we use return fields parameter in
    #       reference documents get_all()

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def id(self) -> str:
        return str(self.data["id"])

    @property
    def name(self) -> str:
        return str(self.data["name"])

    @property
    def tags(self) -> list[str]:
        return self.data["tags"]

    @property
    def metadata(self) -> str:
        return str(self.data["metadata"])

    @property
    def filename(self) -> str:
        return str(self.data["filename"])

    @property
    def created(self) -> int:
        return int(self.data["created"])

    @property
    def updated(self) -> int:
        return int(self.data["updated"])

    @property
    def processed(self) -> int:
        return bool(self.data["processed"])

    @property
    def lang(self) -> Optional[str]:
        return self.data.get("documentName", None)

    @property
    def content(self) -> Optional[str]:
        return self.data.get("content", None)

    @property
    def document_class(self) -> Optional[Entity]:
        return self.data.get("documentClass", None)

    @property
    def derived_tags(self) -> list[str]:
        return self.data["derivedTags"]

    @property
    def color(self) -> Optional[str]:
        return self.data.get("color", None)

    @property
    def derived_color(self) -> Optional[str]:
        return self.data.get("derivedColor", None)

    @property
    def comment(self) -> Optional[str]:
        return self.data.get("comment", None)

    @property
    def derived_comment(self) -> Optional[str]:
        return self.data.get("derivedComment", None)


@dataclass(frozen=True)
class _DocsPerTag(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def tag(self) -> str:
        return str(self.data["tag"])

    @property
    def count(self) -> int:
        return int(self.data["count"])


@dataclass(frozen=True)
class Statistic(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def library_size(self) -> int:
        return int(self.data["librarySize"])

    @property
    def size(self) -> int:
        return int(self.data["size"])

    @property
    def number_of_sentences(self) -> int:
        return int(self.data["numberOfSentences"])

    @property
    def docs_per_tag(self) -> list[_DocsPerTag]:
        return [
            _DocsPerTag(docs_per_tag) for docs_per_tag in self.data["docsPerTag"]
        ]


@dataclass(frozen=True)
class ReferenceDocumentCollection(SemanthaModelEntity):
    def __post_init__(self):
        assert type(self.data) is list

    def __getitem__(self, index):
        return DocumentInformation(self.data[index])


@dataclass(frozen=True)
class ReferenceDocuments(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def meta(self) -> Meta:
        return Meta(self.data["meta"])

    @property
    def documents(self) -> ReferenceDocumentCollection:
        return ReferenceDocumentCollection(self.data["data"])



from dataclasses import dataclass

from semantha_sdk.model import SemanthaModelEntity
from semantha_sdk.model.DomainSettings import DomainSettings


@dataclass(frozen=True)
class DomainConfiguration(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def id(self) -> str:
        return str(self.data["id"])

    @property
    def name(self) -> str:
        return str(self.data["name"])

    @property
    def settings(self) -> DomainSettings:
        return DomainSettings(self.data["settings"])



from io import IOBase

from semantha_sdk.api import SemanthaAPIEndpoint
from semantha_sdk.model import Metadata


class DocumentComparisons(SemanthaAPIEndpoint):

    @property
    def _endpoint(self):
        return self._parent_endpoint + "/documentcomparisons"

    def post(
            self,
            file: IOBase,
            reference_document: IOBase,
            similarity_threshold: float = 0.85,
            synonymous_threshold: float = 0.98,
            mark_no_match: bool = True,
            with_reference_text: bool = True,
            document_type: str = None,
            metadata: list[Metadata] = None,
            with_context: bool = True,
            consider_text_type: bool = False,
            resize_paragraphs: bool = False

    ):
        """ Determine references (for temporary data)

        Args:
            file (IOBase): Input document (left document).
            reference_document (Document): Reference document(s) to be used instead of the documents in the domain's library.
            similarity_threshold (float): Threshold for the similarity score.
                semantha will not deliver results with a sentence score lower than the threshold.
                In general, the higher the threshold, the more precise the results.
            synonymous_threshold (float): Threshold for good matches.
            mark_no_match (bool): Marks paragraphs that have not matched.
            with_reference_text (bool): Provide the reference text in the result JSON.
                If set to false, you have to query the library to resolve the references yourself.
            document_type (str): Specifies the document type that is to be used by semantha when reading the uploaded PDF document.
            metadata (list[Metadata]): Filter by metadata
            with_context (bool): Creates and saves the context.
            consider_text_type (bool): Use this parameter to ensure that only paragraphs of the same type are compared with each other.
                The parameter is of type boolean and is set to false by default.
            resize_paragraphs (bool): Automatically resizes paragraphs based on their semantic meaning
        """

        return self._session.post(
            self._endpoint,
            body={
                "file": file,
                "referencedocument": reference_document.data,
                "similaritythreshold": str(similarity_threshold),
                "synonymousthreshold": str(synonymous_threshold),
                "marknomatch": str(mark_no_match),
                "withreferencetext": str(with_reference_text),
                "documenttype": document_type,
                "metadata": metadata,
                "withcontext": str(with_context),
                "considertexttype": str(consider_text_type),
                "resizeparagraphs": str(resize_paragraphs)
            }
        ).execute()




from .SemanthaModelEntity import SemanthaModelEntity
from .References import Reference
from .Paragraphs import Paragraph
from .Pages import Page
from .Domains import Domains
from .Class import Class
from .DataType import DataType
from .Label import Label
from .Metadata import Metadata
from .DocumentMetadata import DocumentMetadata



from dataclasses import dataclass
from typing import Optional

from semantha_sdk.model import Reference, SemanthaModelEntity
from semantha_sdk.model.Sentences import Sentence


@dataclass(frozen=True)
class Paragraph(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def id(self) -> str:
        return self.data["id"]

    @property
    def type(self) -> str:
        return self.data["type"]

    @property
    def text(self) -> str:
        return self.data["text"]

    @property
    def sentences(self) -> list[Sentence]:
        return [
            Sentence(raw_sentence) for raw_sentence in self.data["sentences"]
        ]

    @property
    def document_name(self) -> Optional[str]:
        # TODO IDK if that name is correct
        return (
            self.data["documentName"]
            if "documentName" in self.data
            else None
        )

    @property
    def references(self) -> Optional[list[Reference]]:
        return [
            Reference(raw_sentence)
            for raw_sentence in self.data.get("references", [])
        ]

    @property
    def context(self) -> dict[str, str]:
        return None

    @property
    def area(self):
        return None

    @property
    def comment(self) -> str:
        return None

    @property
    def data_url_imag(self) -> str:
        return None

    context: dict[str, str] = None
    # TODO: areas DTO
    areas = None
    comment: str = None
    data_url_imag: str = None

    # TODO: error handling
    # TODO: type hints



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['semantha_sdk',
 'semantha_sdk.api',
 'semantha_sdk.model',
 'semantha_sdk.request',
 'semantha_sdk.response',
 'semantha_sdk.rest']

package_data = \
{'': ['*']}

install_requires = \
['requests==2.28.1']

setup_kwargs = {
    'name': 'semantha-sdk',
    'version': '4.2.0',
    'description': 'This is a python client sdk for accessing semantha (the semantic platform)',
    'long_description': '![](https://www.semantha.de/wp-content/uploads/semantha-inverted.svg)\n\n# semantha® SDK\n\nThe semantha SDK is a high-level REST client to access the [semantha](http://semantha.ai) API.\nThe SDK is still under development.\nAn overview of the current progress (i.e. implemented and tested resources and endpoints) may be found at the end of this document (State of Development).\n\n### Disclaimer\n**IMPORTANT:** The SDK is under development and interfaces may change at any time without notice. Use with caution and on own risk.\n\n### Access\nTo access semantha\'s API you will need an API and a server url.\nBoth can be requested via [this contact form](https://www.semantha.de/request/).\n\n\n\n### Basic Usage\n\n#### Import\n```\nimport semantha_sdk\n```\n#### Authentication\n```\nsemantha = semantha_sdk.login(url="<semantha platform server URL>", key="<your key>")\n# or\nsemantha = semantha_sdk.login(url="<semantha platform server URL>", key_file="<path to your key file (json format)>")\n```\n#### End-point Access\n```\n# end-points (resp. resources) can be used like objects\ncurrent_user = semantha.current_user\nmy_domain = semantha.domains.get_one("my_domain")\n\n# they may have sub-resources, which can be retrieved as objects as well\nreference_documents = my_domains.reference_documents\n```\n#### CRUD on End-points\n```\n# CRUD operations are functions\ndomain_settings = my_domain.get_settings()\nmy_domain.reference_documents.delete_all()\n```\n#### Function Return Types & semantha Data Model\n```\n# some functions only return None, e.g.\nmy_domain.reference_documents.delete_all() # returns NoneType\n\n# others return built in types, e.g\nroles_list = current_user.get_user_roles() # returns list[str]\n\n# but most return objects of the semantha Data Model\n# (all returned objects are instances of frozen dataclasses)\nsettings = my_domain.get_settings() # returns instance of DomainSettings\n# attributes can be accessed as properties, e.g.\nsettings.enable_tagging # returns true or false\n# Data Model objects may be complex\ndocument = my_domain.references.post(file=a, reference_document=b) # returns instance of Document\n# the following returns the similarity value of the first references of the first sentence of the\n# the first paragraph on the first page of the document (if a reference was found for this sentence)\nsimilarity = pages[0].contents[0].paragraphs[0].references[0].similarity # returns float\n```\n\n### State of Development\nThe following resources and end-points are fully functional and (partially) tested:\nCurrentUser\n* get_user_data\n* get_user_roles\n\nDomains\n* get_all\n* get_one --> returns sub-resource Domain\n\t* get_configuration\n\t* get_settings\n\t* References\n\t\t* post\n\t* ReferenceDocuments\n\t\t* get_all\n\t\t* get_one\n\t\t* delete_all\n\t\t* delete_one\n\t\t* post\n\t\t* get_paragraph\n\t\t* delete_paragraph\n\t\t* get_sentence\n\t\t* get_named_entities\n\t\t* get_statistic\n\nModel/Domains\n* get_one --> returns sub-resource DomainModel\n\t* Boostwords\n\t\t* get_all\n\t\t* get_one\n\t\t* delete_all\n\t\t* get_one\n',
    'author': 'Tom Kaminski',
    'author_email': 'tom.kaminski@semantha.de',
    'maintainer': 'semantha support',
    'maintainer_email': 'support@semantha.de',
    'url': 'https://semantha.de',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import json
from typing import Dict

from requests import Response
from requests.structures import CaseInsensitiveDict

from semantha_sdk.model import SemanthaModelEntity
from semantha_sdk.response.SemanthaPlatformError import SemanthaPlatformError


class SemanthaPlatformResponse:
    def __init__(self, response: Response) -> None:
        self.__raw_response = response
        self.__content = _check_and_translate_errors(self.__raw_response).content

    def get_status_code(self) -> int:
        return self.__raw_response.status_code

    # TODO: we should parse the Dict and return something more simple
    def get_headers(self) -> CaseInsensitiveDict[str]:
        return self.__raw_response.headers

    def as_dict(self) -> Dict:
        response_as_dict = json.loads(self.__content.decode())
        if type(response_as_dict) is dict:
            return response_as_dict
        else:
            raise ValueError(f"Response content could not be converted to type 'dict':\n{self.__str__()}")

    def as_dict_or_none(self):
        try:
            return self.as_dict()
        except ValueError:
            return None

    def as_list(self) -> list:
        response_as_list = json.loads(self.__content.decode())
        if type(response_as_list) is list:
            return response_as_list
        else:
            raise ValueError(f"Response content could not be converted to type 'list':\n{self.__str__()}")

    def as_list_or_none(self):
        try:
            return self.as_list()
        except ValueError:
            return None

    # TODO: type hint via interface
    def to(self, dto_type):
        try:
            as_dict = self.as_dict_or_none()
            if as_dict is not None:
                return dto_type(as_dict)
            as_list = self.as_list_or_none()
            if as_list is not None:
                return dto_type(as_list)
        except ValueError:
            raise ValueError(f"The content of the semantha platform server response could not be converted to type "
                             f"'{dto_type.__str__()}'")

    def __str__(self) -> str:
        return self.__raw_response.__str__()


def _check_and_translate_errors(response: Response) -> Response:
    if response.status_code in [200, 204]:
        return response
    else:
        if response.status_code == 400:
            raise ValueError(response.status_code, response.text)
        elif response.status_code == 401:
            raise PermissionError(response.status_code, response.text)
        elif response.status_code == 404:
            raise FileNotFoundError(response.status_code, response.text)
        elif response.status_code == 405:
            raise NotImplementedError(response.status_code, response.text)
        elif response.status_code == 408:
            raise TimeoutError(response.status_code, response.text)
        elif response.status_code == 500:
            raise SemanthaPlatformError(response.status_code, response.text)
        # elif response.status_code == 503:
        #     raise BusyError(response.status_code, response.text)
        else:
            raise RuntimeError(response.status_code, response.text)

    # TODO: Method for finding out the response type



from dataclasses import dataclass

from semantha_sdk.model import SemanthaModelEntity


@dataclass(frozen=True)
class Entity(SemanthaModelEntity):

    def __post_init__(self):
        assert type(self.data) is dict

    @property
    def id(self) -> str:
        return str(self.data["id"])

    @property
    def name(self) -> str:
        return str(self.data["name"])



from semantha_sdk.api import SemanthaAPIEndpoint
from semantha_sdk.api.DomainModel import Domains
from semantha_sdk.rest.RestClient import RestClient


class Model(SemanthaAPIEndpoint):

    def __init__(self, session: RestClient, parent_endpoint: str):
        super().__init__(session, parent_endpoint)
        self.__domains = Domains(session, self._endpoint)

    @property
    def _endpoint(self):
        return self._parent_endpoint + "/model"

    @property
    def domains(self) -> Domains:
        return self.__domains



import json

from semantha_sdk.rest.RestClient import RestClient
from semantha_sdk.api.SemanthaAPI import SemanthaAPI


def login(server_url: str, key: str = None, key_file: str = None) -> SemanthaAPI:
    """ Access the Semantha API.

    Args:
        server_url (str): URL to the Semantha server
        key (str): A valid bearer token for accessing the given url.
        key_file (str): Path to a json file providing a valid `API_Key` value for the given url.

    Returns:
        SemanthaAPI: Entry point to the Semantha API.
    """
    __s = None
    if key:
        __s = SemanthaAPI(RestClient(server_url, key), "/api")
    elif key_file:
        with open(key_file, "r") as key_file:
            __s = SemanthaAPI(RestClient(server_url, json.load(key_file)['API_Key']), "/api")
    else:
        raise ValueError("You need to supply an API key to login, either directly or via a file.")

    # check whether API key is valid or not
    __s.current_user.get_user_data()

    return __s



from semantha_sdk.api import SemanthaAPIEndpoint
from semantha_sdk.response import SemanthaPlatformResponse


class Diff(SemanthaAPIEndpoint):
    """ Create diffs between two texts. """

    @property
    def _endpoint(self):
        return self._parent_endpoint + "/diff"

    def create_diff(self, left_text: str, right_text: str) -> SemanthaPlatformResponse:
        """ Create a diff between two given texts.

        Args:
            left_text (object): One of the two texts for the diff.
            right_text (object): The other text for the diff.
        """

        return self._session.post(
            self._endpoint,
            {
                "left": left_text,
                "right": right_text
            }
        ).execute()
